/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FieldMetadataConfigurator.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.configurator.metadata;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.configurator.BasicConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.AnnotationVisitorDispatcher;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificClassConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificFieldConfigurator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.JAnnotation;

/**
 * FieldMetadataConfigurator is a configurator of field specific to FieldMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class FieldMetadataConfigurator<
    C extends IClassMetadata<C, M, F>,
    M extends IMethodMetadata<C, M, F>,
    F extends IFieldMetadata<C, M, F>>
        extends BasicConfigurator implements IFieldConfigurator {

    /**
     * the field metadata.
     */
    private F fieldMetadata;

    /**
     * List of specific field configurator.
     */
    private List<ISpecificFieldConfigurator> specificFieldConfiguratorList;

    /**
     * @return the field metadata
     */
    protected F getFieldMetadata() {
        return fieldMetadata;
    }

    /**
     * Construtor.
     * @param fieldMetadata the field metadata
     */
    public FieldMetadataConfigurator(final F fieldMetadata) {
        this.fieldMetadata = fieldMetadata;
        specificFieldConfiguratorList = new ArrayList<ISpecificFieldConfigurator>();
    }

    /**
     * Construtor.
     * @param fieldMetadata the field metadata
     * @param specificClassConfiguratorList a list of specific class configurator
     */
    public FieldMetadataConfigurator(final F fieldMetadata,
            final List<ISpecificClassConfigurator<C, M, F>> specificClassConfiguratorList) {
        this.fieldMetadata = fieldMetadata;
        specificFieldConfiguratorList = new ArrayList<ISpecificFieldConfigurator>();
        if (specificClassConfiguratorList != null) {
            for (ISpecificClassConfigurator<C, M, F> specificClassConfigurator : specificClassConfiguratorList) {
                specificFieldConfiguratorList.add(specificClassConfigurator.createSpecificFieldConfigurator(fieldMetadata));
            }
        }
    }

    /**
     * @param annotation an annotation
     * @return a visitor for this annotation
     */
    @Override
    public IAnnotationVisitor getAnnotationVisitor(final JAnnotation annotation) {
        List<IAnnotationVisitor> annotationVisitorList = new ArrayList<IAnnotationVisitor>();
        IAnnotationVisitor annotationVisitor = super.getAnnotationVisitor(annotation);
        if (annotationVisitor != null) {
            annotationVisitorList.add(annotationVisitor);
        }
        for (ISpecificFieldConfigurator specificFieldConfigurator : specificFieldConfiguratorList) {
            annotationVisitor = specificFieldConfigurator.getAnnotationVisitor(annotation);
            if (annotationVisitor != null) {
                annotationVisitorList.add(annotationVisitor);
            }
        }
        switch(annotationVisitorList.size()) {
        case 0:
            return null;
        case 1:
            return annotationVisitorList.get(0);
        default:
            return new AnnotationVisitorDispatcher(annotationVisitorList);
        }
    }


    /**
     * Add the field metadata to his class metadata when configuration ended.
     * @param referencedClasses you may fill this set with class referenced in current metadata
     */
    @Override
    public void configurationComplete(final Set<String> referencedClasses) {
        fieldMetadata.getClassMetadata().addStandardFieldMetadata(fieldMetadata);
        for (ISpecificFieldConfigurator specificFieldConfigurator : specificFieldConfiguratorList) {
            specificFieldConfigurator.configurationComplete(referencedClasses);
        }
        super.configurationComplete(referencedClasses);
    }

}
